import VueI18Next from '@panter/vue-i18next'
import i18next from 'i18next'
import Fetch from 'i18next-fetch-backend'
import LocalStorageBackend from 'i18next-localstorage-backend'
import Vue from 'vue'
import VueRouter from 'vue-router'
import vuetify_cs from 'vuetify/es5/locale/cs'
import vuetify_en from 'vuetify/es5/locale/en'
import vuetify_colors from 'vuetify/es5/util/colors'
import Vuex, { Store as VuexStore } from 'vuex'
import YAML from 'yaml'

import { Store } from 'FerdaJS/store'
import PageNotFound from 'ferda/common/PageNotFound.vue'

Vue.use(Vuex)
Vue.use(VueI18Next)
Vue.use(VueRouter)

// Setup default settings for router and extend them at module level
export const router = new VueRouter({ mode: 'history', routes: [{ path: '*', component: PageNotFound }] })

export const initialize_store = vuex_store => {
    const user_profile_el = document.querySelector('#user-profile')
    if (user_profile_el) {
        vuex_store.commit('general/set_user', JSON.parse(user_profile_el.textContent))
    }

    const user_permissions_el = document.querySelector('#user-permissions')
    if (user_permissions_el) {
        vuex_store.commit('general/set_permissions', JSON.parse(user_permissions_el.textContent))
    }

    const django_settings_el = document.querySelector('#django-settings')
    if (django_settings_el) {
        const settings = JSON.parse(django_settings_el.textContent)
        vuex_store.commit('settings/set_time_zone', settings.TIME_ZONE)
        vuex_store.commit('settings/set_language_code', settings.LANGUAGE_CODE)
        vuex_store.commit('settings/set_languages', settings.LANGUAGES)
        vuex_store.commit('settings/set_fido2_authentication', settings.FIDO2_AUTHENTICATION)
        vuex_store.commit('settings/set_manual_in_zone_default_duration', settings.MANUAL_IN_ZONE_DEFAULT_DURATION)
        vuex_store.commit('settings/set_object_detail_logger_services', settings.OBJECT_DETAIL_LOGGER_SERVICES)
    }
}

export const store = new VuexStore(Store)

export const get_vuetify_options = () => ({
    lang: {
        locales: { en: vuetify_en, cs: vuetify_cs },
        current: store.getters['preferences/selected_language']
            || store.getters['settings/LANGUAGE_CODE']
            || 'en',
    },
    theme: {
        themes: {
            dark: {
                accent: vuetify_colors.blue.lighten4,
                action_add: vuetify_colors.grey.darken3,
                action_anchor: vuetify_colors.grey.darken3,
                action_delete: vuetify_colors.grey.darken3,
                action_edit: vuetify_colors.grey.darken3,
                action_export: vuetify_colors.grey.darken3,
                action_refresh: vuetify_colors.grey.darken3,
                action_search: vuetify_colors.grey.darken3,
                action_toggle: vuetify_colors.grey.darken3,
                primary: vuetify_colors.blue.base,
                secondary: vuetify_colors.amber.base,
                tooltip_bg: '#000000',
            },
            light: {
                accent: vuetify_colors.blue.lighten4,
                action_add: vuetify_colors.grey.darken3,
                action_anchor: vuetify_colors.grey.darken3,
                action_delete: vuetify_colors.grey.darken3,
                action_edit: vuetify_colors.grey.darken3,
                action_export: vuetify_colors.grey.darken3,
                action_refresh: vuetify_colors.grey.darken3,
                action_search: vuetify_colors.grey.darken3,
                action_toggle: vuetify_colors.grey.darken3,
                primary: vuetify_colors.blue.base,
                secondary: vuetify_colors.amber.base,
                tooltip_bg: '#000000',
            },
        },
    },
    options: {
        customProperties: true,
    },
})

const ferda_version_el = document.querySelector('#ferda-version')
export const ferda_version = ferda_version_el ? JSON.parse(ferda_version_el.textContent).version : 'no_version'

export const i18n = new VueI18Next(i18next)

export const get_i18next_options = () => ({
    lng: store.getters['preferences/selected_language'],
    fallbackLng: [store.getters['settings/LANGUAGE_CODE'], 'en'],
    ns: [
        'common',
        'notifications',
        'logger',
        'login',
        'search',
        'contact',
        'domain',
        'nsset',
        'keyset',
        'registrar',
        'rule',
        'message',
        'report',
    ],
    defaultNS: 'common',
    backend: {
        backends: [LocalStorageBackend, Fetch],
        backendOptions: [
            {
                versions: store.getters['settings/LANGUAGES'].reduce((language_versions, lang) => {
                    language_versions[lang[0]] = ferda_version
                    return language_versions
                }, {}),
            },
            {
                loadPath: '/static/ferda/locales/{{lng}}/{{ns}}.yml',
                requestOptions: {
                    cache: 'reload',
                },
                parse: data => YAML.parse(data),
            },
        ],
    },
})
